"""Utilities for creating a command line processor.
   Copyright 2010 by genForma Corporation. Licensed under LGPL 3.
"""

from optparse import OptionParser
import sys
import os.path

class Command(object):
    """Base class for packager commands. Subclases should define the following class properties:
          NAME       - name of the command
          USAGE      - e.g. "%prog [options] foo arg1 arg2"
          SHORT_DESC - short (less than a line) description, printed by --help
          LONG_DESC  - longer description, printed by help command
          
    """
    def __init__(self, parser, args, options, num_additional_args=None):
        """Subclass constructors should call this one and then parse their arguments. If there
        is a problem, they should call parser.error(). We do a check here in the base
        case that the number of additional args is correct."""
        self.parser = parser
        self.args = args # the command has been stripped off the args array
        self.options = options
        # check number of additional ares if specified
        if (num_additional_args != None) and (len(args) != num_additional_args):
            parser.error("Expecting %d arguments for %s command." % (num_additional_args, self.get_name()))

    def get_name(self):
        return self.__class__.NAME

    def run(self):
        """Subclasses should implement this method. Returns an exit code
        """
        pass


class _CommandHelpCommand(Command):
    """Command to print help about other commands"""
    NAME = "help"
    USAGE = "%prog help command_name"
    SHORT_DESC = "Print more detailed help about a specific command"
    LONG_DESC = "Prints usage and description of specified command, then exits."
    def __init__(self, parser, args, options, num_additional_args=None):
        super(_CommandHelpCommand, self).__init__(parser, args, options, 1)
        self.commands = None # set by command manager
        self.requested_command = args[0]

    def set_commands(self, commands):
        self.commands = commands
        
    def run(self):
        if not self.commands.has_key(self.requested_command):
            sys.stderr.write("No command named '%s'\n" % self.requested_command)
            return 2
        command = self.commands[self.requested_command]
        usage = command.USAGE.replace("%prog", os.path.basename(sys.argv[0]))
        print "Usage:\n %s\n\n%s\n\n" % (usage, command.SHORT_DESC)
        print "%s\n" % command.LONG_DESC
        return 0
        


class CommandManager(object):
    def __init__(self):
        self.commands = {}
        self.command_names = []
        self.register_command(_CommandHelpCommand)

    def register_command(self, command):
        """command should be a subclass of Command whose constructor expects
           the following arguments: parser, args, options.
        """
        self.commands[command.NAME] = command
        self.command_names.append(command.NAME)

    def get_usage(self):
        cmd_str = "\n".join(["{cmd: <15} {desc}".format(cmd=self.commands[cn].NAME, desc=self.commands[cn].SHORT_DESC) for cn in self.command_names])
        return "%prog [options] COMMAND [ARGS]\n\nCOMMAND is one of:\n" + cmd_str + "\n"

    def add_options(self, parser):
        """This is for subclasses to add more options if desired.
        """
        pass

    def process_generic_options(self, options, args):
        """This is for subclasses to process any generic options before
        handing processing to the specified command.
        """
        pass
    
    def parse_command_args(self, argv):
        """Parse the command line arguments and instantiate the requested
        command.
        """
        parser = OptionParser(usage=self.get_usage())
        self.add_options(parser)
        (options, args) = parser.parse_args()
        self.process_generic_options(options, args)
        # determine the command, with error checking
        if len(args)<1:
            parser.error("First argument must be command, which is one of %s"
                         % ", ".join(self.command_names))
        command_name = args[0]
        if command_name not in self.command_names:
            parser.error("Command '%s' not valid, must be one of %s" %
                         (command_name, ", ".join(self.command_names)))
        command_constructor = self.commands[command_name]
        command = command_constructor(parser, args[1:], options)
        if command.get_name() == _CommandHelpCommand.NAME:
            # need a special case to pass the command map to the help command
            command.set_commands(self.commands)
        return command




"""
This is the Engage SDK for Django applications.

The SDK provides a command line utility to determine in advance whether your
Django application can be deployed on Engage and to package it up for
Engage. If you just want to get up and running quickly, take a look at the
Quickstart guide (the readme.rst file associated with this package). For
more detailed documentation, see the Engage Django SDK reference manual
on the genForma website.

This Engage SDK is Copyright 2010,2011 by genForma Corporation.

"""



"""Version of the SDK
"""


# The following line should have VERSION, an equal sign, and the version number in single quotes.
# There should be no whitespace. This makes it easy to automatically extract the version number
VERSION='0.7.0'



"""Miscellaneous utility functions"""

import os
import os.path
import json
import re

def app_module_name_to_dir(app_directory_path, app_module_name, check_for_init_pys=True):
    """The application module could be a submodule, so we may need to split each level"""
    dirs = app_module_name.split(".")
    dirpath = app_directory_path
    module_name = None
    for dirname in dirs:
        if module_name:
            module_name = module_name + "." + dirname
        else:
            module_name = dirname
        dirpath = os.path.join(dirpath, dirname)
        init_file = os.path.join(dirpath, "__init__.py")
        if check_for_init_pys and not os.path.exists(init_file):
            raise ValidationError("Missing __init__.py file for module %s" % module_name)
    return dirpath


def write_json(json_obj, filename):
    f = open(filename, 'wb')
    json.dump(json_obj, f)
    f.close()


def find_files(directory, filename_re_pattern, operation_function):
    """Find all the files recursively under directory whose names contain the specified pattern
       and run the operation_function on the fileame.
    """
    regexp = re.compile(filename_re_pattern)
    directory = os.path.abspath(os.path.expanduser(directory))
    for root, dirs, files in os.walk(directory):
        for filename in files:
            if regexp.search(filename):
                operation_function(os.path.join(root, filename))


def get_deployed_settings_module(django_settings_module):    
    mod_comps = django_settings_module.split('.')
    if len(mod_comps)==1:
        return "deployed_settings"
    else:
        return '.'.join(mod_comps[0:-1]) + ".deployed_settings"




class ValidationError(Exception):
    """Base class for all expected errors that we find. Don't print a stack trace
    for these.
    """
    pass

class FileFormatError(ValidationError):
    pass

class FileMissingError(ValidationError):
    pass

class ArchiveStructureError(ValidationError):
    pass

class SettingsImportError(ValidationError):
    """Error when trying to import the django settings file"""
    pass

class TestSetupError(ValidationError):
    """This exception is thrown when there is a problem with setting up the tests,
    not necessarily with the application itself.
    """
    pass

class VersionError(ValidationError):
    """This error is signaled when the version of the packager used to package the app is
    obsolete.
    """
    pass


import json
import os.path

from errors import FileFormatError, VersionError
import utils


class DjangoConfig(object):
    def __init__(self, product, product_version, django_settings_module, python_path_subdirectory, version,
                 installed_apps=[], fixtures=[], migration_apps=[]):
        self.product = product
        self.product_version = product_version
        self.django_settings_module = django_settings_module
        # python_path_subdirectory is the directory which should be set as the PYTHONPATH,
        # relative from the top level directory of the application archive.
        # If the PYTHONPATH should point to the parent directory of the application
        # archive, then python_path_subdirectory will be the empty string.
        self.python_path_subdirectory = python_path_subdirectory
        self.installed_apps = installed_apps
        self.fixtures = fixtures
        self.version = version # version of the SDK
        self.migration_apps = migration_apps

    def to_json(self):
        obj = {
            "product":self.product,
            "product_version":self.product_version,
            "django_settings_module":self.django_settings_module,
            "python_path_subdirectory":self.python_path_subdirectory,
            "version":self.version,
            "installed_apps":self.installed_apps,
            "fixtures":self.fixtures,
            "migration_apps":self.migration_apps
            }
        return obj

    def get_settings_module_file(self):
        """Return the path to the settings file. This is a relative path,
        from the start of the application directory. Thus, the first component
        of this path will always be the common directory of the application archive.
        """
        if self.python_path_subdirectory != "":
            return os.path.join(self.python_path_subdirectory,
                                '/'.join(self.django_settings_module.split('.'))) + ".py"
        else:
            return '/'.join(self.django_settings_module.split('.')) + ".py"

    def get_settings_file_directory(self, app_parent_dir):
        """Given the directory above the one where we are extracting the app,
        return the directory containing the settings file.
        """
        return os.path.dirname(os.path.join(app_parent_dir, self.get_settings_module_file()))

    def get_deployed_settings_module(self):
        # deployed settings module is the one we generate and use as the actual django settings module
        return utils.get_deployed_settings_module(self.django_settings_module)

    def get_python_path_directory(self, app_parent_dir):
        """This is the directory we need to add to PYTHONPATH
        """
        if self.python_path_subdirectory == "":
            return app_parent_dir
        else:
            return os.path.join(app_parent_dir, self.python_path_subdirectory)


def compatible_versions(current_version, compatible_version):
    """We check the first two components of the version. The current version
    must be equal to or greater than the compatible version in both components.
    Ill-formed version numbers result in a False return value.

    >>> compatible_versions("1.0.0", "1.0.0")
    True
    >>> compatible_versions("1.0.0", "1.0.1")
    True
    >>> compatible_versions("1.0.1", "1.0.0")
    True
    >>> compatible_versions("1.1.0", "1.0.0")
    True
    >>> compatible_versions("1.0.0", "1.1.0")
    False
    >>> compatible_versions("2.0.0", "1.0.0")
    True
    >>> compatible_versions("2.0.0", "2.1.0")
    False
    >>> compatible_versions("2.0.0", "2.0.5")
    True
    >>> compatible_versions("2.0.0", "1.9")
    True
    """
    version_list = current_version.split(".")
    compat_list = compatible_version.split(".")
    assert len(compat_list) >= 2, "Compabile version number should have at least two components"
    if len(version_list)<2 or (version_list[0]< compat_list[0]) or \
        (version_list[0]==compat_list[0] and version_list[1]<compat_list[1]):
        return False
    else:
        return True


def django_config_from_json_obj(json_obj, compatible_version):
    # First, we get the version property, which must always be there. We
    # check that first, as the presence of the other properties may depend
    # on the version, and we don't want to give a confusing error message
    # (see ticket #163).
    if not json_obj.has_key("version"):
        raise FileFormatError("django engage configuration file missing required key 'version'")
    version = json_obj["version"]
    if not compatible_versions(version, compatible_version):
        raise VersionError("Application package was built using an obsolete version of the SDK (%s). Please download the latest version and repackage your application." %
                           version)
    for key in ["product", "product_version", "django_settings_module", "python_path_subdirectory", "installed_apps", "fixtures", "migration_apps"]:
        if not json_obj.has_key(key):
            raise FileFormatError("django engage configuration file missing required key %s" % key)
    return DjangoConfig(json_obj['product'],
                        json_obj['product_version'],
                        json_obj['django_settings_module'],
                        json_obj['python_path_subdirectory'], json_obj['version'],
                        json_obj['installed_apps'], json_obj['fixtures'],
                        migration_apps=json_obj['migration_apps'])

def django_config_from_json(json_string, compatible_version):
    try:
        json_obj = json.loads(json_string)
    except ValueError, m:
        raise FileFormatError("Unable to parse django engage configuration file: %s" % m)
    return django_config_from_json_obj(json_obj, compatible_version)



import re
def parse_requirements(fp):
    requirements = []
    dependencies = []
    for line in fp.read().split('\n'):
        if re.match(r'(\s*#)|(\s*$)', line):
            continue
        if re.match(r'\s*-r\s+', line):
            dependencies.append(re.sub(r'\s*-r\s+', '', line))
        elif re.match(r'\s*-f\s+', line):
            requirements.append(re.sub(r'\s*-f\s+[^\s]*\s+([^\>\<\=]*)[\>|\>\=|\=\=](.*)$', r'\1', line))
        elif re.match(r'\s*--find-links\s*', line):
            requirements.append(re.sub(r'\s*--find-links\s+[^\s]*\s+([^\>\<\=]*)[\>|\>\=|\=\=](.*)$', r'\1', line))
        elif re.match(r'\s*-e\s+', line):
            requirements.append(re.sub(r'\s*-e\s+.*#egg=(.*)$', r'\1', line))
        elif re.match(r'(.*)[\>|\>\=|\=\=](.*)$', line):
            requirements.append(re.sub(r'([^\>\=\<]*)[\>|\>\=|\=\=](.*)$', r'\1', line))
        elif re.match(r'\s*index-url\s*', line):
            print 'Warning: --index-url not supported yet.'
            pass
        else:
            requirements.append(line)
    return (requirements, dependencies)

def parse_dependency_links(fp):
    dependency_links = []
    for line in fp.read().split('\n'):
        if re.match(r'\s*-[ef]\s+', line):
            dependency_links.append(re.sub(r'\s*-[ef]\s+', '', line))
    return dependency_links

def parse(fname):
    packages = []
    fset = set()
    files = [fname]
    while (files):
        f = files.pop()
        if f in fset:
            print 'Warning: circular dependency in requirements.txt file: %s' % f
            break
        fset.add(f)
        try:
            fp = open(f, 'r')
        except IOError:
            print 'Warning: requirements file %s not found for reading' % f
            break
        (reqs, deps) = parse_requirements(fp)
        fp.close()
        packages = packages + reqs
        files = files + deps
    return packages

if __name__ == '__main__':
    packages = parse('requirements.txt') 
    print packages



#! /usr/bin/env python

import sys
import os
import os.path
from subprocess import Popen
import traceback
from optparse import OptionParser
import tempfile
import shutil
import json
import logging

from generate_settings import simulated_install
from validation_results import SETTINGS_VALIDATION_ERROR_RC, SUCCESS_RC
from __init__ import run_installed_tests_as_subprocess


logger = logging.getLogger()

verbose = False

class TestException(Exception):
    pass


def system(command):
    """Run a command in the shell. If the command fails, we thow an
    exception.
    """
    p = Popen(command, shell=True)
    (pid, exit_status) = os.waitpid(p.pid, 0)
    rc = exit_status >> 8 # the low byte is the signal ending the proc
    if rc != 0:
        raise TestException("Command execution failed: '%s'" % command)


PACKAGE_MAIN=os.path.abspath(os.path.dirname(__file__))
PYTHON_EXE=sys.executable

def run_packager(args, expected_rc=0):
    """Run the packager in a subprocess. If the command does not exit with the
       expected return code, we thow an exception.
    """
    if verbose:
        command = "%s %s -v %s" % (PYTHON_EXE, PACKAGE_MAIN, " ".join(args))
    else:
        command = "%s %s %s" % (PYTHON_EXE, PACKAGE_MAIN, " ".join(args))
    p = Popen(command, shell=True)
    (pid, exit_status) = os.waitpid(p.pid, 0)
    rc = exit_status >> 8 # the low byte is the signal ending the proc
    if rc != expected_rc:
        raise TestException("Packager execution failed: '%s', rc was %s, expecting %s" % (command, rc, expected_rc))


def _run_validation_tests(archive_file, app_module_name, tmpdir):
        run_packager(["validate-safe", archive_file])
        run_packager(["validate-full", archive_file])
        # for the validate-installed case, we need to simulate the installation
        (install_path, undo_ops) = simulated_install(archive_file, tmpdir,
                                                     app_module_name)
        results_file = os.path.join(tmpdir, "results.json")
        run_packager(["validate-installed", install_path, results_file])
        assert os.path.exists(results_file)
        fileobj = open(results_file, "rb")
        json_obj = json.load(fileobj)
        fileobj.close()
        if type(json_obj)!=dict:
            raise TestException("Invalid results file, contents are %s" %
                                json_obj.__repr__())
        os.remove(results_file)
        parsed_results = run_installed_tests_as_subprocess(install_path,
                                                           app_module_name,
                                                           use_logger=logger)
        if parsed_results.get_return_code()!=SUCCESS_RC:
            if parsed_results.has_error_messages():
                logger.debug("parsed results bad, messages = %s" %
                             parsed_results.format_messages())
            raise TestException("run as subprocess test failed, rc was %d" %
                                parsed_results.get_return_code())

    
def test_directory(dir_path, app_module_name, options):
    tmpdir = tempfile.mkdtemp(suffix="-engage-test")
    try:
        archive_file = os.path.join(tmpdir, "test_archive.tgz")
        if options.prepare_negative_test:
            run_packager(["prepare-dir", dir_path, app_module_name, archive_file],
                         expected_rc=SETTINGS_VALIDATION_ERROR_RC)
            logger.info("Negative tests for prepare-dir successful.")
        else:
            run_packager(["prepare-dir", dir_path, app_module_name, archive_file])
            _run_validation_tests(archive_file, app_module_name, tmpdir)
            logger.info("Tests for prepare-dir successful.")
        return 0
    finally:
        shutil.rmtree(tmpdir)


def test_archive(archive_file, app_module_name, options):
    tmpdir = tempfile.mkdtemp(suffix="-engage-test")
    try:
        if options.prepare_negative_test:
            run_packager(["prepare-archive", archive_file, app_module_name],
                         expected_rc=SETTINGS_VALIDATION_ERROR_RC)
            logger.info("Negative tests for prepare-archive successful.")
        else:
            run_packager(["prepare-archive", archive_file, app_module_name])
            _run_validation_tests(archive_file, app_module_name, tmpdir)
            logger.info("Tests for prepare-archive successful.")
        return 0
    finally:
        shutil.rmtree(tmpdir)



def main(argv):
    try:
        parser = OptionParser(usage="%prog [options] django_archive_file django_settings_module\n%prog [options] django_app_directory django_settings_module")
        parser.add_option("-n", "--prepare-negative-test", action="store_true",
                          dest="prepare_negative_test", default=False,
                          help="Prepare validation checks should fail")
        parser.add_option("-v", "--verbose", action="store_true",
                          dest="verbose", default=False,
                          help="Print debug output for packager")
        (options, args) = parser.parse_args()
        if len(args)!=2:
            parser.error("Expecting two arguments")
        global verbose
        verbose = options.verbose
        if verbose:
            LOG_LEVEL = logging.DEBUG
        else:
            LOG_LEVEL = logging.INFO

        logger.setLevel(LOG_LEVEL)
        handler = logging.StreamHandler(sys.stdout)
        handler.setLevel(LOG_LEVEL)
        logger.addHandler(handler)

        if os.path.isdir(args[0]):
            rc = test_directory(args[0], args[1], options)
        elif os.path.exists(args[0]):
            rc = test_archive(args[0], args[1], options)
        else:
            parser.error("File or directory %s does not exist" % args[0])
        return rc
    except TestException:
        (exc_type, exc_value, exc_traceback) = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, None, file=sys.stderr)
        logger.error("Tests failed due to TestException")
        return 1
    except SystemExit, c:
        sys.exit(c)
    except:
        (exc_type, exc_value, exc_traceback) = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, exc_traceback, file=sys.stderr)
        logger.error("Tests failed due to unexpected exception of type %s, value was %s" % (exc_type, exc_value))
        return 1


if __name__ == "__main__":
    sys.exit(main(sys.argv))



"""Tracking and representation of settings validation results"""

import json

# Return codes
SUCCESS_RC                   = 0
SETTINGS_VALIDATION_ERROR_RC = 1
ARG_PARSE_ERROR_RC           = 2
ARCHIVE_VALIDATION_ERROR_RC  = 3
UNEXPECTED_EXC_RC            = 4
INTERNAL_ERROR_RC            = 5

_rc_descriptions = {
    SUCCESS_RC: "Success",
    SETTINGS_VALIDATION_ERROR_RC: "Settings Validation Errors",
    ARG_PARSE_ERROR_RC: "Command Line Agument Error",
    ARCHIVE_VALIDATION_ERROR_RC: "Archive Validation Error",
    UNEXPECTED_EXC_RC: "Unexpected Exception",
    INTERNAL_ERROR_RC: "InternalError"
}

class ResultsBase(object):
    def __init__(self, version, error_messages=None, warning_messages=None,
                 exception=None):
        self.version = version
        if error_messages:
            self.error_messages = error_messages
        else:
            self.error_messages = []
        if warning_messages:
            self.warning_messages = warning_messages
        else:
            self.warning_messages = []
        self.exception = exception

    def get_return_code(self):
        pass

    def get_return_code_desc(self):
        if _rc_descriptions.has_key(self.get_return_code()):
            return  _rc_descriptions[self.get_return_code()]
        else:
            return "Unexpected Return Code"
        
    def to_json(self):
        js = { u"validator_version":self.version,
               u"return_code": self.get_return_code(),
               u"return_code_desc": self.get_return_code_desc(),
               u"error_messages": self.error_messages,
               u"warning_messages": self.warning_messages,
               u"exception": self.exception }
        return js


class ParsedJsonResults(ResultsBase):
    """This object represents the results file after it has been
    read back in and parsed. Most of the work is done in the __init__
    function which converts from a dictionary to object fields.
    """
    def __init__(self, json_obj):
        try:
            if not type(json_obj)==dict:
                raise Exception("Invalid content in results file, data was %s" %
                                json_obj.__repr__())
            props = [u"return_code", u"return_code_desc",
                     u"validator_version", u"error_messages",
                     u"warning_messages", u"exception"]
            for prop in props:
                if not json_obj.has_key(prop):
                    raise Exception("Invalid results file, missing property %s" %
                                    prop)
            ResultsBase.__init__(self,
                                 json_obj[u"validator_version"],
                                 json_obj[u"error_messages"],
                                 json_obj[u"warning_messages"],
                                 json_obj[u"exception"])
            self.return_code = json_obj[u"return_code"]
            self.return_code_desc = json_obj[u"return_code_desc"]
        except Exception, v:
            ResultsBase.__init__(self, "Unknown", [ v ], [])
            self.return_code = INTERNAL_ERROR_RC
            self.return_code_desc = _rc_descriptions[INTERNAL_ERROR_RC]

    def get_return_code(self):
        return self.return_code
    
    def run_was_successful(self):
        return self.return_code == SUCCESS_RC
    
    def get_return_code_desc(self):
        return self.return_code_desc

    def get_validator_version(self):
        return self.validator_version

    def has_error_messages(self):
        return len(self.error_messages) > 0

    def get_error_messages(self):
        return self.error_messages

    def has_warning_messages(self):
        return len(self.warning_messages) > 0

    def get_warning_messages(self):
        return self.warning_messages

    def format_messages(self):
        s = ""
        if self.has_error_messages():
            s += "Errors:\n"
            for i in range(1, len(self.error_messages)+1):
                s += "  [%02d] %s\n" % (i, self.error_messages[i-1])
        else:
            s += "Errors: None\n"
        if self.has_warning_messages():
            s += "Warnings:\n"
            for i in range(1, len(self.warning_messages)+1):
                s += "  [%02d] %s\n" % (i, self.warning_messages[i-1])
        else:
            s += "Warnings: None\n"
        return s

    def has_exception(self):
        return self.exception


class SettingValidationResults(ResultsBase):
    """This class is used for tracking the progress of the settings validation tests.
    """
    def __init__(self, version, logger):
        ResultsBase.__init__(self, version)
        self.logger = logger
        self.product = None
        self.product_version = None
        self.python_path_subdirectory = None
        self.installed_apps = None
        self.fixtures = None
        self.migration_apps = None
        self.errors = 0
        self.warnings = 0

    def error(self, msg):
        self.errors += 1
        self.error_messages.append(msg)
        self.logger.error("ERROR: " + msg + "\n")

    def warning(self, msg):
        self.warnings += 1
        self.warning_messages.append(msg)
        self.logger.warn("WARNING: " + msg + "\n")

    def has_errors(self):
        return self.errors > 0

    def has_warnings(self):
        return self.warnings > 0

    def all_checks_passed(self):
        return (self.errors == 0) and (self.warnings == 0)

    def print_final_status_message(self, logger):
        if self.all_checks_passed():
            self.logger.info("All settings checks passed\n")
        else:
            self.logger.error("%d errors, %d warnings in settings validation.\n" %
                              (self.errors, self.warnings))
        
    def get_return_code(self):
        if self.has_errors(): return SETTINGS_VALIDATION_ERROR_RC
        else: return SUCCESS_RC # if we get warnings but not errors, still return ok



import os
import os.path
import unittest

def find_python_module(qualified_module_name, root_dir):
    """This utility function looks for a python module
    under the specified directory. It returns the full
    path to the directory above the module. This directory can
    be added to sys.path to successfully import the module. If
    the module is not found, we return None.

    There is a special case for when the root directory is the
    first (topmost) component of the module. In this case, we return
    the directory above the root directory.
    """
    def check_module_file(parent_dir, module_name):
        module_file = os.path.join(parent_dir, module_name + ".py")
        if os.path.exists(module_file):
            return True
        else:
            return False
    
    def check_dir(current_dir, module_list):
        """Return True if the module_list exists in the current directory"""
        if len(module_list)==1:
            return check_module_file(current_dir, module_list[0])
        else:
            next_dir = os.path.join(current_dir, module_list[0])
            if os.path.isdir(next_dir) and os.path.exists(os.path.join(next_dir, "__init__.py")):
                return check_dir(next_dir, module_list[1:])
            else:
                return False

    full_module_list = qualified_module_name.split('.')
    for (dirname, subdirs, files) in os.walk(os.path.abspath(os.path.expanduser(root_dir))):
         module_path = check_dir(dirname, full_module_list)
         if module_path:
             return dirname
    # if we get here, the module isn't under root_dir. we have a special case for when
    # current_dir is the top directory of the module.
    if check_dir(os.path.dirname(root_dir), full_module_list):
        return os.path.dirname(root_dir)
    else:
        return None


class TestFindPythonModule(unittest.TestCase):
    def _make_py_file(self, parent_dir, name):
        with open(os.path.join(parent_dir, name), "w") as f:
            f.write("# \n")

    def setUp(self):
        import tempfile
        self.temp_dir = tempfile.mkdtemp()
        self.w_dir = os.path.join(self.temp_dir, "w")
        os.mkdir(self.w_dir)
        self.x_dir = os.path.join(self.w_dir, "x")
        os.mkdir(self.x_dir)
        self._make_py_file(self.x_dir, "__init__.py")
        self.y_dir = os.path.join(self.x_dir, "y")
        os.mkdir(self.y_dir)
        self._make_py_file(self.y_dir, "__init__.py")
        self._make_py_file(self.y_dir, "z.py")

    def tearDown(self):
        import shutil
        shutil.rmtree(self.temp_dir)

    def test_find_from_temp_dir(self):
        """Check for x.y.z from temp_dir. This should return tmpdir/w.
        """
        res = find_python_module("x.y.z", self.temp_dir)
        self.assertEqual(res, self.w_dir)

    def test_find_from_w(self):
        """Check for x.y.z from w. This should return tmpdir/w.
        """
        res = find_python_module("x.y.z", self.w_dir)
        self.assertEqual(res, self.w_dir)

    def test_find_from_x(self):
        """Check for x.y.z from x. This should return tmpdir/w
        """
        res = find_python_module("x.y.z", self.x_dir)
        self.assertEqual(res, self.w_dir)

    def test_find_z_from_y(self):
        """Check for z from y. This should return tmpdir/w/x/y.
        """
        res = find_python_module("z", self.y_dir)
        self.assertEqual(res, self.y_dir)

    def test_find_from_y(self):
        """Check for x.y.z from y. This should return None.
        """
        res = find_python_module("x.y.z", self.y_dir)
        self.assertEqual(res, None)

    def test_find_bogus_from_w(self):
        """Check for a.b.c from temp_dir. This should return None"""
        res = find_python_module("a.b.c", self.temp_dir)
        self.assertEqual(res, None)


if __name__ == '__main__':
    unittest.main()



#! /user/bin/env python

"""Command line interface to engage-django sdk.
"""

import sys
import os.path
from optparse import OptionParser
import logging
import traceback

import packager
import packager.command
import packager.errors

logger = logging.getLogger()

class CommandManager(packager.command.CommandManager):
    def __init__(self):
        super(CommandManager, self).__init__()
        self.verbose = False

    def add_options(self, parser):
        parser.add_option("-v", "--verbose", action="store_true", dest="verbose")

    def process_generic_options(self, options, args):
        self.verbose = options.verbose

command_manager = CommandManager()


## class PrintDocsCommand(packager.command.Command):
##     NAME = "print-docs"
##     USAGE = "%prog print-docs"
##     SHORT_DESC = "Prints the packager documentation and exits"
##     LONG_DESC = "Prints to standard output."
##     def __init__(self, parser, args, options):
##         super(PrintDocsCommand, self).__init__(parser, args, options, 0)

##     def run(self):
##         sys.stdout.write("Django SDK for Engage\n")
##         sys.stdout.write("=====================================\n")
##         self.parser.print_help()
##         # the user documentation is in the SDK module's docstring
##         sys.stdout.write(sys.modules["engage_django_sdk"].__doc__)
##         sys.exit(0)

## command_manager.register_command(PrintDocsCommand)


command_manager.register_command(packager.PrepareDirCommand)
command_manager.register_command(packager.PrepareArchiveCommand)


def main(argv=sys.argv):
    try:
        (major_v, minor_v, micro_v, releaselevel, serial) = sys.version_info
        if major_v != 2 or minor_v < 6:
            sys.stderr.write("This program only works with Python 2.x (2.6 and later)\n")
            sys.exit(2)
        command = command_manager.parse_command_args(argv)
        root_logger = logging.getLogger()
        handler = logging.StreamHandler()
        root_logger.addHandler(handler)
        if command_manager.verbose:
            root_logger.setLevel(logging.DEBUG)
            handler.setLevel(logging.DEBUG)
        else:
            root_logger.setLevel(logging.INFO)
            handler.setLevel(logging.INFO)            
        rc = command.run()
        logger.info("Command %s completed successfully." % command.get_name())
        return rc
    except packager.errors.ValidationError:
        (exc_type, exc_value, exc_traceback) = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, None, file=sys.stderr)
        return 1
    except SystemExit, c:
        sys.exit(c)
    except:
        (exc_type, exc_value, exc_traceback) = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, exc_traceback, file=sys.stderr)
        return 3


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# Django application validation utility
# Copyright 2010 by genForma Corporation

import sys
import traceback
import logging

from __init__ import *

def main(argv):
    try:
        command = command_manager.parse_command_args(argv)
        root_logger = logging.getLogger()
        handler = logging.StreamHandler()
        root_logger.addHandler(handler)
        if verbose:
            root_logger.setLevel(logging.DEBUG)
            handler.setLevel(logging.DEBUG)
        else:
            root_logger.setLevel(logging.INFO)
            handler.setLevel(logging.INFO)            
        rc = command.run()
        return rc
    except ValidationError:
        (exc_type, exc_value, exc_traceback) = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, None, file=sys.stderr)
        if command.get_results_file():
            write_json(create_results_json(ARCHIVE_VALIDATION_ERROR_RC, command.get_name(), error_messages=[exc_value],
                                            exception=traceback.format_exception(exc_type, exc_value, exc_traceback)),
                        command.get_results_file())
        return ARCHIVE_VALIDATION_ERROR_RC
    except SystemExit, c:
        sys.exit(c)
    except:
        (exc_type, exc_value, exc_traceback) = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, exc_traceback, file=sys.stderr)
        if command.get_results_file():
            write_json(create_results_json(UNEXPECTED_EXC_RC, command.get_name(),
                                             exception=traceback.format_exception(exc_type, exc_value, exc_traceback)),
                        command.get_results_file())
        return UNEXPECTED_EXC_RC 


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python
# Django application packager utility
# Copyright 2010 by genForma Corporation






import sys
import os
import os.path
import shutil
from optparse import OptionParser
import traceback
import string
from copy import deepcopy
import json
import subprocess
import tempfile
import logging
import urlparse

logger = logging.getLogger(__name__)

try:
    from engage_django_sdk.version import VERSION
except ImportError:
    sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../..")))
    from engage_django_sdk.version import VERSION


# We store VERSION in the django_config.json file. COMPATIBLE_PACKAGER_VERSION is the earliest
# version of the packager which is supported by this version of the code. run_safe_validations()
# compares the version stored in the config file against the compatible version. If they config
# file version is older than the compatible version, an exception is thrown. This prevents someone
# from uploading an archive based on an obsolute version of the packager.  Bump up the compatible
# version number when the contents of the config file or the way settings are handled is changed.
#
# Note that we only compare the first two elements of the version number. The third element is
# not considered, so it is ok if the application's version is behind in path level, if the major
# and minor versions match.
COMPATIBLE_PACKAGER_VERSION = ".".join(VERSION.split('.')[0:2])


from errors import *
from archive_handlers import ZipfileHandler, TarfileHandler, create_handler, \
                             validate_archive_files
from django_config import DjangoConfig, django_config_from_json
from generate_settings import generate_settings_file, simulated_install
from utils import app_module_name_to_dir, write_json, find_files, \
                  get_deployed_settings_module
from validation_results import *
import command
import parse_requirements
from find_module import find_python_module

verbose = False


SETTINGS_FILENAME = "settings.py"
DJANGO_CFG_FILENAME = "engage_django_cfg.json"


def run_safe_validations(handler):
    """This function runs all the validations that can safely be done without running
    user code. It returns the common parent directory of the archive.
    """
    # check archive format
    namelist = handler.get_namelist()
    common_dir = validate_archive_files(namelist)
    logger.debug("Common directory is %s" % common_dir)
    
    # check for and then read configuration file
    config_file = common_dir + os.sep + DJANGO_CFG_FILENAME
    if not handler.is_file_in_archive(config_file):
        raise FileMissingError("Archive missing configuration file %s" %
                               config_file)
    fileobj = handler.open_member(config_file)
    data = fileobj.read()
    django_config = django_config_from_json(data, COMPATIBLE_PACKAGER_VERSION)

    # check for other required files
    settings_module_file = django_config.get_settings_module_file()
    if not handler.is_file_in_archive(settings_module_file):
        raise FileMissingError("Archive missing django settings file %s" % settings_module_file)
    logger.debug("Has required files")
    return (common_dir, django_config)


def check_if_setting_overridden(setting_name, module, results):
    if not hasattr(module, setting_name):
        raise ValidationError("Expected setting '%s' not defined. Is the setting file set up correctly?" % setting_name)
    setting_value = getattr(module, setting_name)
    engage_value = getattr(module.engage_settings, setting_name)
    if setting_value != engage_value:
        results.warning("Setting '%s' was overriden to value '%s'" %
                        (setting_name, setting_value))


def check_url_setting(setting_name, setting_value, results, default_value='',
                      path_should_end_in_slash=True):
    if setting_value == default_value:
        logger.debug("Setting '%s' has default value" % setting_name)
        return
    try:
        sr = urlparse.urlsplit(setting_value)
        if path_should_end_in_slash and (sr[2][len(sr[2])-1] != '/'):
            results.warning("Setting '%s': The path component of the URL ('%s') is expected by Django to end in a slash ('/'). Engage will add a slash" %
                            (setting_name, sr[2]))
        logging.debug("Setting '%s' is a valid URL" % setting_name)
    except:
        results.error("Invalid URL format for setting '%s': '%s'" %
                      (setting_name, setting_value))


def check_directory_setting(setting_name, setting_value, default_value,
                            app_dir_path, results):
    if setting_value == default_value:
        logger.debug("Setting '%s' has default value" % setting_name)
    elif not os.path.isdir(setting_value):
        results.error("Setting '%s' points to directory '%s', which does not exist" %
                      (setting_name, setting_value))
    elif string.find(os.path.abspath(setting_value),
                     os.path.abspath(app_dir_path)) != 0:
        results.error("Setting '%s' points to directory '%s', which is not a subdirectory of '%s'" %
                      (setting_name, setting_value, app_dir_path))
    else:
        logger.debug("Setting '%s' passed directory checks." % setting_name)


def check_directory_tuple_setting(setting_name, setting_value,
                                  app_dir_path, results):
    """Validate a setting that is a tuple of directory names.
    """
    if isinstance(setting_value, str) or isinstance(setting_value, unicode):
        results.warning("Expecting a tuple for %s setting, got a string. If only one element, put a comma after the string." %
                        setting_name)
        check_directory_setting(setting_name, setting_value, None,
                                app_dir_path, results)
    else:
        # check each path in the list
        for dirname in setting_value:
            check_directory_setting(setting_name, dirname,
                                    None, app_dir_path, results)


PREINSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.admindocs",
    "django.contrib.auth",
    "django.contrib.comments",
    "django.contrib.contenttypes",
    "django.contrib.csrf",
    "django.contrib.databrowse",
    "django.contrib.flatpages",
    "django.contrib.formtools",
    "django.contrib.gis",
    "django.contrib.humanize",
    "django.contrib.localflavor",
    "django.contrib.markup",
    "django.contrib.messages",
    "django.contrib.redirects",
    "django.contrib.sessions",
    "django.contrib.sitemaps",
    "django.contrib.sites",
    "django.contrib.syndication",
    "django.contrib.webdesign",
    "south"
]

def validate_installed_app(app_name, python_path_dir, known_apps, results):
    if app_name in known_apps:
        logger.debug("Application '%s' Ok." % app_name)
        return
    app_dirs = app_name.split(".")
    current_dir = python_path_dir
    for dirname in app_dirs:
        current_dir = os.path.join(current_dir, dirname)
        if not os.path.exists(current_dir):
            results.warning("Application '%s' not found." % app_name)
            return
    logger.debug("Application '%s' Ok." % app_name)

def validate_post_install_commands(app_name, settings_module, results):
    """Validate that each named command in ENGAGE_DJANGO_POSTINSTALL_COMMANDS
    is actually present as a command in manage.py.
    """
    from django.core.management import get_commands
    commands = get_commands().keys()
    for cmd in results.post_install_commands:
        if cmd not in commands:
            results.error("Unable to find post-install command %s in available commands" % cmd)
            return
    logger.debug("Post installation commands for application %s Ok." % app_name) 

def validate_fixture_file(fixture_name, installed_apps, fixture_dirs,
                          python_path_dir, settings_file_directory, results):
    """Use the same logic as django-admin.py loaddata to search for a
    fixture file.
    """
    if fixture_name.endswith((".json", ".xml")):
        filenames = [fixture_name]
    else:
        filenames = [fixture_name+".json", fixture_name+".xml"]
    # first try an absolute path
    if os.path.isabs(fixture_name):
        for filename in filenames:
            if os.path.exists(filename):
                logging.debug("Found fixture file %s" % filename)
                return
        results.error("Unable to find fixture %s" % fixture_name)
        return
    # next, look in the fixtures directory under all the installed apps
    for app_module in installed_apps:
        if app_module in PREINSTALLED_APPS:
            continue # we can't check the django apps fixtures
        fixture_dir = os.path.join(app_module_name_to_dir(python_path_dir,
                                                          app_module, False),
                                   "fixtures")
        for filename in filenames:
            fixture_path = os.path.join(fixture_dir, filename)
            if os.path.exists(fixture_path):
                logging.debug("Found fixture %s at %s" % (fixture_name, fixture_path))
                return
    # next, look under all the specified fixture directories
    for dirname in fixture_dirs:
        for filename in filenames:
            fixture_path = os.path.join(dirname, filename)
            if os.path.exists(fixture_path):
                logging.debug("Found fixture %s at %s" % (fixture_name, fixture_path))
                return
    # finally, look relative to settings_file_directory
    for filename in filenames:
        fixture_path = os.path.join(settings_file_directory, filename)
        if os.path.exists(fixture_path):
            logging.debug("Found fixture %s at %s" % (fixture_name, fixture_path))
            return
    # if we got here, we didn't find the fixture file anywhere
    results.error("Unable to find fixture %s" % fixture_name)


def validate_migration_apps(migration_apps, installed_apps, results):
    for app in migration_apps:
        found = False
        for installed_app in installed_apps:
            # look for an exact match or a suffix
            if (app == installed_app) or (installed_app[len(installed_app)-len(app)-1:] == ("." + app)):
                logging.debug("Found migration app %s" % app)
                found = True
                break
        if not found:
            results.error("Unable to find migration app %s in installed apps" % app)


def _tuple_setting_to_list(v):
    """We need to have a special case if the user specified a single element
    tuple as (v) instead of (v,).
    """
    if hasattr(v, "__iter__"):
        return list(v)
    else:
        return [v]

def get_user_required_packages(app_dir_path):
    """Look for a requirements.txt file. If found, parse the file and return
    a list of eggs.
    """
    requirements_file = os.path.join(app_dir_path, "requirements.txt")
    if os.path.exists(requirements_file):
        logging.debug("Found requirements.txt file")
        return parse_requirements.parse(requirements_file)
    else:
        logging.debug("Did not find requirements.txt file")
        return []

def _get_subdir_component(parent_dir, subdir):
    """Given two paths, where the second is a subdirectory of the
    first, return the part of the second path, which, when os.path.join()'ed to
    the first path, creates the second path. If the two paths are equal,
    return ''
    """
    parent_dir = os.path.abspath(os.path.expanduser(parent_dir))
    subdir = os.path.abspath(os.path.expanduser(subdir))
    assert subdir[:len(parent_dir)] == parent_dir, \
           "%s not a subdirectory of %s" % (subdir, parent_dir)
    if parent_dir==subdir:
        return ""
    else:
        return subdir[len(parent_dir)+1:]


def validate_settings(app_dir_path, django_settings_module, django_config=None):
    """This is the main settings validation function. It takes the following arguments:
        app_dir_path           - path to the top level directory of the extracted application
        django_settings_module - fully qualified name of the django settings module
        django_config          - if provided, this is the django_config data generated
                                 during the original packaging of the app. We validate
                                 that it is still consistent with the current app.

    This function returns an intance of SettingValidationResults.

    Note that validate_settings() must be run after generate_settings(). We import
    the deployed_settings module rather than the user's django_settings_module so
    we can see whether they've overridden any of the settings.
    """
    # normalize the target install path
    app_dir_path = os.path.abspath(os.path.expanduser(app_dir_path))
    
    results = SettingValidationResults(VERSION, logger)
    python_path_dir = find_python_module(django_settings_module, app_dir_path)
    if not python_path_dir:
        raise ValidationError("Unable to find django settings module %s under %s" % (django_settings_module, app_dir_path))
    # we store only the subdirectory part of the python path, since the rest depends
    # on where we install the app.
    if os.path.dirname(app_dir_path)==python_path_dir:
        results.python_path_subdirectory = ""
    else:
        results.python_path_subdirectory = _get_subdir_component(os.path.dirname(app_dir_path), python_path_dir)
    # get the settings file directory
    settings_module_comps = django_settings_module.split(".")
    if len(settings_module_comps)==1:
        settings_file_directory = python_path_dir
    else:
        settings_file_directory = os.path.join(python_path_dir, "/".join(settings_module_comps[0:-1]))

    # do the import of app's settings
    sys.path = [python_path_dir] + sys.path

    deployed_settings_module = get_deployed_settings_module(django_settings_module)
    logger.debug("Importing settings module %s" % deployed_settings_module)
    try:
        parent_settings_module = __import__(deployed_settings_module)
        settings_module = getattr(parent_settings_module, "deployed_settings")
    except:
        (exc_type, exc_value, exc_traceback) = sys.exc_info()
        logger.error("Exception in settings file import: %s" %
                     traceback.format_exception(exc_type, exc_value, exc_traceback))
        raise SettingsImportError("Error in settings import: %s, %s" % (exc_type, exc_value))
        
    # Check that the settings controlled by engage weren't overridden by app.
    # If any are overridden, we treat them as warnings.
    check_if_setting_overridden('TIME_ZONE', settings_module, results)
    check_if_setting_overridden('SECRET_KEY', settings_module, results)
    check_if_setting_overridden('ADMINS', settings_module, results)
    check_if_setting_overridden('DATABASES', settings_module, results)
    check_if_setting_overridden('LOGGING_CONFIG', settings_module,
                                results)

    # Check that settings which point to a directory are either not set or
    # point to a valid directory
    if hasattr(settings_module, "MEDIA_ROOT"):
        check_directory_setting("MEDIA_ROOT",
                                settings_module.MEDIA_ROOT,
                                '', app_dir_path, results)
    if hasattr(settings_module, "STATIC_MEDIA_ROOT"):
        check_directory_setting("STATIC_MEDIA_ROOT",
                                settings_module.STATIC_MEDIA_ROOT,
                                '', app_dir_path, results)
    if hasattr(settings_module, "TEMPLATE_DIRS"):
        check_directory_tuple_setting("TEMPLATE_DIRS",
                                      settings_module.TEMPLATE_DIRS,
                                      app_dir_path, results)

    # check url settings
    if hasattr(settings_module, "MEDIA_URL"):
        check_url_setting("MEDIA_URL", settings_module.MEDIA_URL,
                          results)
    if hasattr(settings_module, "ADMIN_MEDIA_PREFIX"):
        check_url_setting("ADMIN_MEDIA_PREFIX",
                          settings_module.ADMIN_MEDIA_PREFIX,
                          results)

    # check that all INSTALLED_APPS are pointing to apps accessible in the target system
    if hasattr(settings_module, "INSTALLED_APPS"):
        installed_apps = []
        known_apps = set(PREINSTALLED_APPS + get_user_required_packages(app_dir_path))
        for app_name in settings_module.INSTALLED_APPS:
            validate_installed_app(app_name, python_path_dir, known_apps, results)
            installed_apps.append(app_name)
    else:
        installed_apps = []
    results.installed_apps = installed_apps

    if hasattr(settings_module, "FIXTURE_DIRS"):
        fixture_dirs = _tuple_setting_to_list(settings_module.FIXTURE_DIRS)
        check_directory_tuple_setting("FIXTURE_DIRS", fixture_dirs,
                                      app_dir_path, results)
    else:
        fixture_dirs = []
    # check that ENGAGE_APP_DB_FIXTURES points to valid fixture files
    if hasattr(settings_module, "ENGAGE_APP_DB_FIXTURES"):
        results.fixtures = _tuple_setting_to_list(settings_module.ENGAGE_APP_DB_FIXTURES)
        for fixture in results.fixtures:
            validate_fixture_file(fixture, results.installed_apps, fixture_dirs,
                                  python_path_dir, settings_file_directory, results)
    else:
        results.fixtures = []

    # check ENGAGE_MIGRATION_APPS, if present
    if hasattr(settings_module, "ENGAGE_MIGRATION_APPS"):
        results.migration_apps = _tuple_setting_to_list(settings_module.ENGAGE_MIGRATION_APPS)
        validate_migration_apps(results.migration_apps, results.installed_apps, results)
    else:
        results.migration_apps = []
    # check each command in ENGAGE_DJANGO_POSTINSTALL_COMMANDS is actually present in manager
    if hasattr(settings_module, "ENGAGE_DJANGO_POSTINSTALL_COMMANDS"):
        results.post_install_commands = list(settings_module.ENGAGE_DJANGO_POSTINSTALL_COMMANDS)
        validate_post_install_commands(app_name, settings_module, results)
    else:
        results.post_install_commands = []

    # extract the product name and version, if present
    if hasattr(settings_module, "ENGAGE_PRODUCT_NAME"):
        results.product = settings_module.ENGAGE_PRODUCT_NAME
    if hasattr(settings_module, "ENGAGE_PRODUCT_VERSION"):
        results.product_version = settings_module.ENGAGE_PRODUCT_VERSION

    # if provided, check that the django_config matches the settings values
    if django_config:
        django_config_ok = True
        if installed_apps != django_config.installed_apps:
            results.error("INSTALLED_APPS in configuration file (%s) does not match INSTALLED_APPS in settings file (%s). Your configuration file is likely out of date. Try re-running prepare." %
                          (django_config.installed_apps.__repr__(),
                           installed_apps.__repr__()))
            django_config_ok = False
        if results.fixtures != django_config.fixtures:
            # TODO: this was originally an error, which caused some issues.
            # See ticket #166.
            results.warning("ENGAGE_APP_DB_FIXTURES in configuration file (%s) does not match value in settings file (%s). If this is not what you expect, your configuration file is likely out of date: try re-running prepare." %
                          (django_config.fixtures.__repr__(),
                           results.fixtures.__repr__()))
            django_config_ok = False
        if results.migration_apps != django_config.migration_apps:
            results.error("ENGAGE_MIGRATION_APPS in configuration file (%s) does not match value in settings file (%s). Your configuration file is likely out of date. Try re-running prepare." %
                          (django_config.migration_apps.__repr__(),
                           results.migration_apps.__repr__()))
            django_config_ok = False
        if results.product and results.product != django_config.product:
            results.error("ENGAGE_PRODUCT_NAME in configuration file (%s) does not match value in settings file (%s). Your configuration file is likely out of date. Try re-running prepare." % (django_config.product, results.product))
            django_config_ok = False
        if results.product_version and results.product_version != django_config.product_version:
            results.error("ENGAGE_PRODUCT_VERSION in configuration file (%s) does not match value in settings file (%s). Your configuration file is likely out of date. Try re-running prepare." % (django_config.product_version, results.product_version))
            django_config_ok = False
        if django_config_ok:
            logger.debug("Verified config file is consistent with settings file")

    return results # all done

        
def run_installed_tests_as_subprocess(app_dir_path, django_settings_module, python_exe_path=sys.executable, use_logger=None):
    """Run the installed mode tests as a separate subprocess, using the installed app's environment.
    A results file is used to communcate back the results of the validation. After the run is complete,
    an instance of ParsedJsonResults is returned
    """
    app_dir_path = os.path.abspath(os.path.expanduser(app_dir_path))
    if not os.path.exists(app_dir_path):
        raise TestSetupError("Django settings file validator: application directory '%s' does not exist" % app_dir_path)
    python_path = find_python_module(django_settings_module, app_dir_path)
    if python_path==None:
        raise TestSetupError("Django settings file validator: unable to find settings module %s under %s" %
                             (django_settings_module, app_dir_path))
    if use_logger == None:
        use_logger = logger
    # we create a temporary file that will contain the results
    tf = tempfile.NamedTemporaryFile(delete=False)
    tf.write("null")
    tf.close()
    script_file = os.path.dirname(os.path.abspath(__file__))
    try:
        program_and_args = [python_exe_path, script_file, "-v",
                            "validate-installed", app_dir_path, tf.name]
        if hasattr(use_logger, "action"):
            use_logger.action(' '.join(program_and_args))
        else:
            use_logger.info(' '.join(program_and_args))
        env = {
            "PYTHONPATH": python_path,
            "DJANGO_SETTINGS_MODULE": "%s" % django_settings_module
        }
        subproc = subprocess.Popen(program_and_args,
                                   env=env, stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.STDOUT, cwd=app_dir_path)
        use_logger.debug("Started program %s, pid is %d" % (program_and_args[0],
                                                            subproc.pid))
        subproc.stdin.close()
        for line in subproc.stdout:
            use_logger.debug("[%d] %s" % (subproc.pid, line))
        subproc.wait()
        use_logger.debug("[%d] %s exited with return code %d" % (subproc.pid,
                                                                 program_and_args[0],
                                                                 subproc.returncode))
        results_file = open(tf.name, "rb")
        json_obj = json.load(results_file)
        results_file.close()
        results = ParsedJsonResults(json_obj)
        return results
    finally:
        os.remove(tf.name)


##### Section: Commands #####

class CommandManager(command.CommandManager):
    def __init__(self):
        super(CommandManager, self).__init__()

    def add_options(self, parser):
        parser.add_option("-v", "--verbose", action="store_true", dest="verbose")

    def process_generic_options(self, options, args):
        global verbose
        verbose = options.verbose

command_manager = CommandManager()

class Command(command.Command):
    """Base class for packager commands. Subclases should define NAME and USAGE class properties as well."""
    def __init__(self, parser, args, options, num_additional_args=None):
        super(Command, self).__init__(parser, args, options,
                                      num_additional_args)

    def get_results_file(self):
        """Return the name of the results file or None if there isn't one"""
        return None


class PrepareCommandBase(Command):
    """Base class for the prepare commands. Provides the main functionality
    for validating and preparing an archive. It expects the self.app_module_name
    member to be defined by the subclasses. This base class defines the following
    members:
     * self.archive_file
     * self.archive_type (either zip or tgz)
     * _validate_and_prepare_dir()
    """
    def __init__(self, parser, args, options, argcnt, archive_file):
        super(PrepareCommandBase, self).__init__(parser, args, options,
                                                 argcnt)
        self.archive_file = archive_file
        if self.archive_file[-4:] == ".zip":
            self.archive_type = "zip"
        elif self.archive_file[-4:]==".tgz" or self.archive_file[-7:]==".tar.gz":
            self.archive_type = "tgz"
        else:
            parser.error("Archive filename '%s' must end in either .zip (zip archive), .tgz, or .tar.gz (gzipped tar archive)" % self.archive_file)        

    def _validate_and_prepare_dir(self, app_dir_path, undo_ops):
        """Called by the subclass run() method after setting up the directory.
        """
        results = validate_settings(app_dir_path, self.django_settings_module)
        if results.get_return_code() != SUCCESS_RC:
            results.print_final_status_message(logger)
            return results.get_return_code()
        if results.product:
            product = results.product
        else:
            product = "Custom Django Application"
        if results.product_version:
            product_version = results.product_version
        else:
            product_version = ""
        config = DjangoConfig(product, product_version,
                              self.django_settings_module,
                              results.python_path_subdirectory, VERSION,
                              installed_apps=results.installed_apps,
                              fixtures=results.fixtures,
                              migration_apps=results.migration_apps)
        write_json(config.to_json(), os.path.join(app_dir_path, DJANGO_CFG_FILENAME))

        # undo the changes we made
        for (op_fun, args) in undo_ops:
            op_fun(*args)
        # delete all the .pyc files
        find_files(app_dir_path, "\.pyc$", os.remove)
        # (re)create the archive
        if self.archive_type == "zip":
            archive = ZipfileHandler(self.archive_file, "w")
        else:
            archive = TarfileHandler(self.archive_file, "w:gz")
        archive.create_new_from_dir(app_dir_path)
        archive.close()
        return SUCCESS_RC
        
_prepare_dir_long_desc = \
"""This command performs validations on a django project, generates the
necessary engage configuration file, removes unneeded files (e.g. .pyc files),
and creates a archive file suitable for deployment to Engage.

The arguments are:
 django_project_directory - directory containing the django app and its
                            dependencies.
 django_settings_module   - name of Python module containing the settings
                            for this app.
 output_archive_file      - name of output archive to be generated. Should end
                            in either .tgz (gzipped tar achive),
                            .tar.gz (gzipped tar archive), or .zip
                            (zip archive).
"""

class PrepareDirCommand(PrepareCommandBase):
    NAME = "prepare-dir"
    USAGE = "%prog [options] prepare-dir django_project_directory django_settings_module output_archive_file"
    SHORT_DESC = "Prepare the specified Django project directory for deployment"
    LONG_DESC = _prepare_dir_long_desc
    def __init__(self, parser, args, options):
        if len(args) != 3:
            parser.error("Expecting three arguments to prepare-dir: django_project_directory, django_settings_module, output_archive_file")
        super(PrepareDirCommand, self).__init__(parser, args, options, 3,
                                                args[2])
        self.django_project_directory = os.path.abspath(args[0])
        if not os.path.isdir(self.django_project_directory):
            parser.error("Django project directory %s does not exist or is not a directory." %
                         self.django_project_directory)
        self.django_settings_module = args[1]

    def run(self):
        tmpdir = tempfile.mkdtemp(suffix='-engage')
        try:
            app_dir_path = os.path.join(tmpdir, os.path.basename(self.django_project_directory))
            shutil.copytree(self.django_project_directory, app_dir_path)
            undo_ops = generate_settings_file(app_dir_path,
                                              self.django_settings_module)
            return self._validate_and_prepare_dir(app_dir_path, undo_ops)
        finally:
            shutil.rmtree(tmpdir)

command_manager.register_command(PrepareDirCommand)


_prepare_archive_long_desc = \
"""This command performs validations on a django project archive file,
generates the necessary engage configuration file, and removes unneeded files
(e.g. .pyc files). Note that changes are performed in-place on the original
archive file.

The arguments are:
 django_archive_file    - archive file containing the django app and its
                          dependencies. Should be either a gzipped tar archive
                          (ending in .tgz or .tar.gz) or a zip archive (ending in
                          .zip).
 django_settings_module - name of Python module containing the settings for this app
"""

class PrepareArchiveCommand(PrepareCommandBase):
    NAME = "prepare-archive"
    USAGE = "%prog [options] prepare-archive django_archive_file app_module_name"
    SHORT_DESC = "Prepare the specified Django project archive for deployment"
    LONG_DESC = _prepare_archive_long_desc
    def __init__(self, parser, args, options):
        if len(args) != 2:
            parser.error("Expecting two arguments to prepare-archive: django_archive file, django_settings_module")
        super(PrepareArchiveCommand, self).__init__(parser, args, options, 2,
                                                    args[0])
        self.django_settings_module = args[1]
        if not os.path.exists(self.archive_file):
            parser.error("Django archive file %s does not exist" %
                         self.archive_file)

    def run(self):
        tmpdir = tempfile.mkdtemp(suffix='-engage')
        try:
            (app_dir_path, undo_ops) = simulated_install(self.archive_file, tmpdir,
                                                         self.django_settings_module)
            return self._validate_and_prepare_dir(app_dir_path, undo_ops)
        finally:
            shutil.rmtree(tmpdir)

command_manager.register_command(PrepareArchiveCommand)

_vs_long_desc = \
"""This command validates a django archive created using prepare-dir or
prepare-archive. It only performs validations which do not involve executing
the Python code in the archive. This include checks that the archive is
readable, verifying that all required files are present, and parsing the
engage config file.
"""

class ValidateSafeCommand(Command):
    NAME = "validate-safe"
    USAGE = "%prog [options] validate-safe django_archive_file"
    SHORT_DESC = "Perform safe validations on the specified django application"
    LONG_DESC = _vs_long_desc
    def __init__(self, parser, args, options):
        super(ValidateSafeCommand, self).__init__(parser, args, options, 1)
        self.django_archive_file = args[0]
        if not os.path.exists(self.django_archive_file):
            parser.error("Django archive file %s does not exist" % self.django_archive_file)
            
    def run(self):
        with create_handler(self.django_archive_file) as handler:
            (common_dir, django_config) = run_safe_validations(handler)
        logger.info("All safe mode checks passed.\n")
        return SUCCESS_RC

command_manager.register_command(ValidateSafeCommand)    


_vf_long_desc = \
"""This command validates a django archive created using prepare-dir or
prepare-archive. It performs all validations, including those which execute
the Python code in the archive. This include all the checks performed by
the validate-safe command, as well as loading the settings and verifying
individual settings values.
"""

class ValidateFullCommand(Command):
    NAME = "validate-full"
    USAGE = "%prog [options] validate-full django_archive_file"
    SHORT_DESC = "Perform all validations on the specified django application"
    LONG_DESC = _vf_long_desc
    def __init__(self, parser, args, options):
        super(ValidateFullCommand, self).__init__(parser, args, options, 1)
        self.django_archive_file = args[0]
        if not os.path.exists(self.django_archive_file):
            parser.error("Django archive file %s does not exist" % self.django_archive_file)
            
    def run(self):
        with create_handler(self.django_archive_file) as handler:
            (common_dir, django_config) = run_safe_validations(handler)
            tmpdir = tempfile.mkdtemp(suffix="-engage")
            try:
                app_dir_path = os.path.join(tmpdir, common_dir)
                logger.debug("Extracting application to '%s'" % app_dir_path)
                handler.extract(tmpdir)
                generate_settings_file(app_dir_path, django_config.django_settings_module)
                results = validate_settings(app_dir_path, django_config.django_settings_module,
                                            django_config)
                results.print_final_status_message(logger)
                return results.get_return_code()
            finally:
                shutil.rmtree(tmpdir)

command_manager.register_command(ValidateFullCommand)


_vi_long_desc = \
"""This command validates an installed django application (e.g. deployed
by Engage). It is run near the end of the deployment process before the
application is started. It performs all validations, including those which
execute the Python code in the archive. This include all the checks performed by
the validate-safe command, as well as loading the settings and verifying
individual settings values.
"""

class ValidateInstalledCommand(Command):
    NAME = "validate-installed"
    USAGE = "%prog validate-installed [options] app_dir_path results_file"
    SHORT_DESC = "Perform all validations on an installed django application"
    LONG_DESC = _vi_long_desc
    def __init__(self, parser, args, options):
        super(ValidateInstalledCommand, self).__init__(parser, args, options, 2)
        self.app_dir_path = args[0]
        self.results_file = args[1]
        if not os.path.isdir(self.app_dir_path):
            parser.error("Django application directory %s does not exist or is not a directory" % self.app_dir_path)
    
    def get_results_file(self):
        return self.results_file

    def run(self):
        config_file = os.path.join(self.app_dir_path, DJANGO_CFG_FILENAME)
        if not os.path.exists(config_file):
            raise FileMissingError("Missing configuration file %s" % config_file)
        with open(config_file, "rb") as fileobj:
            django_config = django_config_from_json(fileobj.read(), COMPATIBLE_PACKAGER_VERSION)        
        results = validate_settings(self.app_dir_path, django_config.django_settings_module,
                                    django_config)
        write_json(results.to_json(), self.results_file)
        results.print_final_status_message(logger)
        return results.get_return_code()    


command_manager.register_command(ValidateInstalledCommand)


##### End Section: Commands #####



import zipfile
import tarfile
import os
import tempfile
import shutil
import logging
logger = logging.getLogger(__name__)

from errors import ArchiveStructureError, FileFormatError

def _is_zip_direntry(name):
    l = len(name)
    if (l > 0) and (name[l-1] == '/'): return True
    else: return False

class BaseArchiveHandler(object):
    def __init__(self):
        pass

    def __enter__(self):
        return self

    def __exit__(self, tp, vl, tb):
        self.close()
        return False

    
class ZipfileHandler(BaseArchiveHandler):
    def __init__(self, filename, mode="r"):
        BaseArchiveHandler.__init__(self)
        self.filename = filename
        if mode=="r":
            self.archive = zipfile.ZipFile(filename, mode)
        else:
            self.archive = zipfile.ZipFile(filename, mode,
                                           compression=zipfile.ZIP_DEFLATED)
        self.mode = mode

    def is_file_in_archive(self, entry_name):
        try:
            info = self.archive.getinfo(entry_name)
            return True
        except KeyError:
            return False

    def get_namelist(self):
        return self.archive.namelist()

    def extract(self, parent_dir):
        """This is a workaround for a bug in Python's ZipFile.extractall() method implementation.
        It does not handle extracting subdirectories correctly, so we extract the files individually,
        creating subdirectories by hand.
        """
        for name in self.archive.namelist():
            if _is_zip_direntry(name):
                subdir = os.path.join(parent_dir, name)
                if not os.path.exists(subdir):
                    os.makedirs(subdir)
            else:
                self.archive.extract(name, parent_dir)

    def open_member(self, member):
        """Open the specified member file for reading, returning a file-like
        object.
        """
        return self.archive.open(member)

    def add_member(self, filepath, path_in_archive):
        assert self.mode=='a' or self.mode=='w'
        self.archive.write(filepath, path_in_archive)
        
    def close(self):
        self.archive.close()

    def create_new_from_dir(self, src_dirpath):
        assert self.mode == "w"
        logger.debug("creating archive file at %s" % self.filename)
        src_dirpath = os.path.abspath(os.path.expanduser(src_dirpath))
        parent_dir = os.path.dirname(src_dirpath)
        for (dirname, subdirs, files) in os.walk(src_dirpath):
            subdir = dirname[len(parent_dir)+1:]
            for file in files:
                filepath = os.path.join(subdir, file)
                self.archive.write(os.path.join(dirname, file), filepath)


class TarfileHandler(BaseArchiveHandler):
    def __init__(self, filename, mode="r"):
        BaseArchiveHandler.__init__(self)
        self.filename = filename
        self.archive = tarfile.open(filename, mode)
        self.mode = mode

    def is_file_in_archive(self, entry_name):
        try:
            info = self.archive.getmember(entry_name)
            return True
        except KeyError:
            return False

    def get_namelist(self):
        return self.archive.getnames()

    def extract(self, parent_dir):
        self.archive.extractall(parent_dir)

    def open_member(self, member):
        """Open the specified member file for reading, returning a file-like
        object.
        """
        return self.archive.extractfile(member)

    def close(self):
        self.archive.close()

    def create_new_from_dir(self, src_dirpath):
        assert self.mode[0] == "w"
        logger.debug("creating archive file at %s" % self.filename)
        src_dirpath = os.path.abspath(os.path.expanduser(src_dirpath))
        self.archive.add(src_dirpath, os.path.basename(src_dirpath))


_dotslash = "." + os.sep
_dotslashlen = len(_dotslash)

def _get_first_subdir_component(path):
    """Helper used in extracting the first subdirectory from an archive toc.
    It expects a relative path.
    """
    if os.path.isabs(path): return None
    if path.find(_dotslash)==0:
        path = path[_dotslashlen:]
    if len(path)==0 or (path=="."): return None
    idx = path.find(os.sep)
    if idx==-1: return path
    else: return path[0:idx]

def validate_archive_files(namelist):
    """Given a list of files in an archive, validate that they are relative paths, not absolute. Also,
    determine whether this is a common directory under which all files appear. If either test fails, raise
    an exception. Otherwise, return the name of the common directory
    """
    common_dirname = None
    for name in namelist:
        if os.path.isabs(name):
            raise ArchiveStructureError("Invalid format for archive: contains absolute file path %s" % name)
        if name.find(os.sep): # contains a directory component
            subdir = _get_first_subdir_component(name)
        else:
            subdir = name
        if common_dirname == None:
            common_dirname = subdir
        elif common_dirname != subdir:
            raise ArchiveStructureError("File %s does not fall under common subdirectory %s" % (name, common_dirname))
    return common_dirname


def create_handler(filename):
    # figure out the type of the file
    if zipfile.is_zipfile(filename):
        logger.debug("File is a zipfile")
        return ZipfileHandler(filename)
    elif tarfile.is_tarfile(filename):
        logger.debug("File is a tarfile")
        return TarfileHandler(filename)
    else:
        raise FileFormatError("Unable to read file %s -- neither a zip file or a tar file" % filename)


def add_file_to_existing_archive(archive_file, new_filename, path_in_archive):
    if zipfile.is_zipfile(archive_file):
        z = ZipfileHandler(archive_file, "a")
        z.add_member(new_filename, path_in_archive)
        z.close()
    elif tarfile.is_tarfile(archive_file):
        # No good way to add a file to an existing tgz archive. We just expand
        # it, add the file, and recreate the archive.
        tmpdir = tempfile.mkdtemp()
        try:
            t = TarfileHandler(archive_file)
            t.extract(tmpdir)
            common_dir = validate_archive_files(t.get_namelist())
            t.close()
            assert path_in_archive[0:len(common_dir)] == common_dir
            copy_dest = os.path.join(tmpdir, path_in_archive)
            shutil.copyfile(new_filename, copy_dest)
            t = TarfileHandler(archive_file, "w:gz")
            t.create_new_from_dir(os.path.join(tmpdir, common_dir))
            t.close()
        finally:
            shutil.rmtree(tmpdir)
    else:
        raise FileFormatError("Unable to read file %s -- neither a zip file or a tar file" % archive_file)



from setuptools import setup, find_packages

from stat import ST_MTIME
import os
import os.path
import sys
from subprocess import Popen

import engage_django_sdk
from engage_django_sdk.version import VERSION



setup(
    name='engage-django-sdk',
    version=VERSION,
    author='genForma Corporation',
    author_email='support@genforma.com',
    packages=find_packages(),
    install_requires = [ "Django>=1.2", ],
    url='http://www.genforma.com',
    license='Apache V2.0',
    entry_points = {
        'console_scripts': [
            'engage-django-sdk = engage_django_sdk.__main__:main'
            ]},
    description='Engage SDK for Django',
    long_description=open('README.rst').read(),
    classifiers=[
    "Environment :: Console",
    "License :: OSI Approved :: Apache Software License",
    "Operating System :: POSIX",
    "Topic :: System :: Systems Administration"
    ],
)



"""Functions related to generating a simulated settings file.
"""

import os
import os.path
import sys
import string
from random import choice
from copy import deepcopy
import unittest
import logging
logger = logging.getLogger(__name__)


from utils import app_module_name_to_dir
from archive_handlers import create_handler, validate_archive_files
from find_module import find_python_module
from errors import FileMissingError

###### Constants ##########
INSTALL_PATH="install_path"
ADMIN_NAME="admin_name"
ADMIN_EMAIL="admin_email"
SECRET_KEY="secret_key"
DATABASE_ENGINE="database_engine"
DATABASE_NAME="database_name"
DATABASE_USER="database_user"
DATABASE_PASSWORD="database_password"
DATABASE_HOST="database_host"
DATABASE_PORT="database_port"
HOSTNAME="hostname"
PRIVATE_IP_ADDRESS="private_ip_address"
PORT="port"
LOG_DIRECTORY="log_directory"
TIME_ZONE="time_zone"
EMAIL_HOST="email_host"
EMAIL_HOST_USER="email_host_user"
EMAIL_FROM="email_from"
EMAIL_HOST_PASSWORD="email_host_password"
EMAIL_PORT="email_port"


################# Start of engage_settings.py template ##############
engage_settings_template = """
# Auto-generated settings values for django

import sys
import os.path
import urlparse

# additional configuration properties exposed by the django driver
ENGAGE_APP_FILES_BASE = '${install_path}'
ENGAGE_WEBSERVER_HOSTNAME = '${hostname}'
ENGAGE_WEBSERVER_PORT = '${port}'
ENGAGE_LOG_DIRECTORY = '${log_directory}'
ENGAGE_PRIVATE_IP_ADDRESS = ${private_ip_address}

if len(sys.argv[0])>0:
    _log_base_name = os.path.basename(sys.argv[0]).replace(".py","")
else:
    _log_base_name = os.path.basename(sys.executable)
if ((_log_base_name=="django-admin") or (_log_base_name=="manage")) and len(sys.argv)>1:
    _log_base_name = _log_base_name + "-" + sys.argv[1]
ENGAGE_LOG_FILE = os.path.join('${log_directory}', '%s.log' % _log_base_name)

# Engage values for Django standard settings
ADMINS = (('${admin_name}', '${admin_email}'),)
DATABASES = {
    'default': {
        'ENGINE' :  '${database_engine}',  # 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME' :  '${database_name}',            # Or path to database file if using sqlite3.
        'USER' : '${database_user}',          # Not used with sqlite3.
        'PASSWORD' : '${database_password}',        # Not used with sqlite3.
        'HOST' : '${database_host}',            # Set to empty string for localhost. Not used with sqlite3.
        'PORT' : '${database_port}',            # Set to empty string for default. Not used with sqlite3.
    }
}
TIME_ZONE = '${time_zone}'
SECRET_KEY = '${secret_key}'

import logging
import logging.handlers
def log_setup_fn(dummy):
    root_logger = logging.getLogger()
    if not os.path.exists(ENGAGE_LOG_DIRECTORY):
        os.makedirs(ENGAGE_LOG_DIRECTORY)
    handler = logging.handlers.RotatingFileHandler(ENGAGE_LOG_FILE, maxBytes=1000000, backupCount=5)
    root_logger.addHandler(handler)
    root_logger.setLevel(logging.INFO)
    handler.setLevel(logging.INFO)            

def dummy_log_setup(dummy):
    pass

LOGGING_CONFIG = '${_logging_fn}'
LOGGING = None


# The following settings are not yet configurable through the website, but
# will eventually be added.
EMAIL_HOST = '${email_host}'
EMAIL_HOST_USER = '${email_host_user}'
EMAIL_FROM = '${email_from}'
EMAIL_HOST_PASSWORD = '${email_host_password}'
EMAIL_PORT = '${email_port}'

# _resolve_name() and import_module() are taken from django.utils.importlib.
# We include them here to enable simulation of the future logging functionality.
# Can remove when we upgrade to the next version of django.
import sys
def _resolve_name(name, package, level):
    if not hasattr(package, 'rindex'):
        raise ValueError("'package' not set to a string")
    dot = len(package)
    for x in xrange(level, 1, -1):
        try:
            dot = package.rindex('.', 0, dot)
        except ValueError:
            raise ValueError("attempted relative import beyond top-level "
                              "package")
    return "%s.%s" % (package[:dot], name)


def import_module(name, package=None):
    if name.startswith('.'):
        if not package:
            raise TypeError("relative imports require the 'package' argument")
        level = 0
        for character in name:
            if character != '.':
                break
            level += 1
        name = _resolve_name(name[level:], package, level)
    __import__(name)
    return sys.modules[name]

def fixup_url(url, ensure_trailing_slash=False):
    sr = urlparse.urlsplit(url)
    if sr[0]=='' and sr[1]=='':
        proto = ''
        netloc = ''
    else:
        proto = 'http'
        if ENGAGE_WEBSERVER_PORT == '80':
            netloc = ENGAGE_WEBSERVER_HOSTNAME
        else:
            netloc = ENGAGE_WEBSERVER_HOSTNAME + ':' + ENGAGE_WEBSERVER_PORT
    if ensure_trailing_slash and (sr[2]!='') and (sr[2][len(sr[2])-1]!='/'):
        path = sr[2] + '/'
    else:
        path = sr[2]
    return urlparse.urlunsplit((proto, netloc, path, sr[3], sr[4]))

"""
################# End of engage_settings.py template ##############

################# Start of deployed_settings.py ##############
deployed_settings_template="""
from ${django_settings_module} import *
_symbols = set(dir())
import ${engage_settings_module} as engage_settings

# the standard django settings that Engage will change at deployment time
if "TIME_ZONE_OVERRIDE" not in _symbols:
    TIME_ZONE = engage_settings.TIME_ZONE

if "SECRET_KEY_OVERRIDE" not in _symbols:
    SECRET_KEY = engage_settings.SECRET_KEY

if "DATABASES_OVERRIDE" not in _symbols:
    DATABASES = engage_settings.DATABASES

if "ADMINS_OVERRIDE" not in _symbols:
    ADMINS = engage_settings.ADMINS

if "LOGGING_OVERRIDE" not in _symbols:
    LOGGING = engage_settings.LOGGING

if "LOGGING_CONFIG_OVERRIDE" not in _symbols:
    LOGGING_CONFIG = engage_settings.LOGGING_CONFIG

# settings that can reference an absolute URL may need to be adjusted.
if ('MEDIA_URL' in _symbols) and (MEDIA_URL != ''):
    MEDIA_URL = engage_settings.fixup_url(MEDIA_URL, True)
if ('ADMIN_MEDIA_PREFIX' in _symbols) and (ADMIN_MEDIA_PREFIX != ''):
    ADMIN_MEDIA_PREFIX = engage_settings.fixup_url(ADMIN_MEDIA_PREFIX, True)

# logging config support not yet in released django,
# so we call it explicitly. This is similar to the latest code
# in django/config/__init__.py
if LOGGING_CONFIG:
    _logging_config_path, _logging_config_func_name = LOGGING_CONFIG.rsplit('.', 1)
    _logging_config_module = engage_settings.import_module(_logging_config_path)
    _logging_config_func = getattr(_logging_config_module, _logging_config_func_name)
    _logging_config_func(LOGGING)
"""
################# End of deployed_settings.py ##############


def _gen_rand_string(length, chars=string.letters+string.digits):
    return ''.join([ choice(chars) for i in range(length) ])


def generate_settings_file(app_dir_path, django_settings_module, properties=None):
    """Generate the engage_settings.py and deployed_settings.py files. Returns a
    list of operations which will put the files back to their original state,
    where each operation is a (function, arglist) tuple.
    """
    undo_ops = []
    # first, get the location of the python path.
    python_path = find_python_module(django_settings_module, app_dir_path)
    if not python_path:
        raise FileMissingError("Cannot find django settings module %s under %s" %
                               (django_settings_module, app_dir_path))

    # compute the paths for the engage_settings.py and deployed_settings.py modules
    subdirs = django_settings_module.split('.')
    if len(subdirs)==1:
        parent_dir = python_path
        engage_settings_module = "engage_settings"
        deployed_settings_module = "deployed_settings"
    else:
        parent_dir = os.path.join(python_path, "/".join(subdirs[0:-1]))
        parent_module = ".".join(subdirs[0:-1])
        engage_settings_module = parent_module + ".engage_settings"
        deployed_settings_module = parent_module + "deployed_settings"
        
    assert os.path.exists(parent_dir), "Internal error: could not find directory %s" % parent_dir
    engage_settings_py = os.path.join(parent_dir, "engage_settings.py")
    deployed_settings_py = os.path.join(parent_dir, "deployed_settings.py")
        

    # warn if the generated files already exist
    if os.path.exists(engage_settings_py):
        logger.warn("%s already exists, will be overwritten" % engage_settings_py)
    if os.path.exists(deployed_settings_py):
        logger.warn("%s already exists, will be overwritten" % deployed_settings_py)
        
    # now generate the engage settings file
    if not properties:
        properties = {
            INSTALL_PATH: os.path.dirname(app_dir_path),
            ADMIN_NAME: "admin_" + _gen_rand_string(5),
            ADMIN_EMAIL: "admn" + _gen_rand_string(5) + "@foo.com",
            SECRET_KEY: _gen_rand_string(50, chars=string.digits+string.letters+"!#%&()*+,-./:;<=>?@[]^_`{|}~"),
            DATABASE_ENGINE: 'django.db.backends.sqlite3',
            DATABASE_NAME: os.path.join(os.path.join(parent_dir, "db"), "django.db"),
            DATABASE_USER: '',
            DATABASE_PASSWORD: '',
            DATABASE_HOST: '',
            DATABASE_PORT: '',
            HOSTNAME: "localhost",
            PRIVATE_IP_ADDRESS: None,
            PORT: 8000,
            LOG_DIRECTORY: os.path.join(app_dir_path, "log"),
            TIME_ZONE: _gen_rand_string(8),
            EMAIL_HOST: 'bar.com',
            EMAIL_HOST_USER: "admin",
            EMAIL_FROM: 'admin@bar.com',
            EMAIL_HOST_PASSWORD: _gen_rand_string(6),
            EMAIL_PORT: 25,
            "_logging_fn": "%s.dummy_log_setup" % \
                           engage_settings_module
        }
    else:
        properties = deepcopy(properties)
        properties["_logging_fn"] = "%s.log_setup_fn" % \
                                    engage_settings_module
    template = string.Template(engage_settings_template)
    result = template.substitute(properties)
    with open(engage_settings_py, "wb") as f:
        f.write(result)
    undo_ops.insert(0, (os.remove, [engage_settings_py]))

    # now generate the deployed_settings.py file
    template = string.Template(deployed_settings_template)
    result = template.substitute({"django_settings_module": django_settings_module,
                                  "engage_settings_module": engage_settings_module})
    with open(deployed_settings_py, "wb") as f:
        f.write(result)
    undo_ops.insert(0, (os.remove, [deployed_settings_py]))
    return undo_ops


def simulated_install(archive_file, install_path, django_settings_module):
    """Simulate the install process. install_path is the directory under
    which we extract the archive.
    """
    with create_handler(archive_file) as h:
        common_dir = validate_archive_files(h.get_namelist())
        h.extract(install_path)
    app_dir_path = os.path.join(install_path, common_dir)
    undo_ops = generate_settings_file(app_dir_path, django_settings_module)
    return (app_dir_path, undo_ops)
    

class TestGenerateSettings(unittest.TestCase):
    """Unit tests for the generate_settings function.
    """
    def _make_py_file(self, parent_dir, name, contents="# \n"):
        with open(os.path.join(parent_dir, name), "w") as f:
            f.write(contents)

    def setUp(self):
        """We set up a sample application directory layout in a temp directory.
        This temp directory is deleted in the tearDown method so that we don't
        leave a mess around.
        """
        import tempfile
        self.temp_dir = tempfile.mkdtemp()
        self.app_dir_path = os.path.join(self.temp_dir, "app_dir")
        os.mkdir(self.app_dir_path)
        self.settings_file_dir = os.path.join(self.app_dir_path, "settings_file_dir")
        os.mkdir(self.settings_file_dir)
        self._make_py_file(self.settings_file_dir, "__init__.py")
        self._make_py_file(self.settings_file_dir, "settings.py")
        self.django_settings_module = "settings_file_dir.settings"
        self.saved_sys_path = sys.path

    def tearDown(self):
        import shutil
        shutil.rmtree(self.temp_dir)
        sys.path = self.saved_sys_path

    def assertFileExists(self, filepath):
        self.assertTrue(os.path.exists(filepath), "%s is missing" % filepath)

    def assertModuleHasSetting(self, module, setting):
        self.assertTrue(hasattr(module, setting), "Module missing setting %s" % setting)

    def test_generate_settings(self):
        ops = generate_settings_file(self.app_dir_path, self.django_settings_module)
        self.assertFileExists(os.path.join(self.settings_file_dir, "engage_settings.py"))
        self.assertFileExists(os.path.join(self.settings_file_dir, "deployed_settings.py"))
        sys.path = [self.app_dir_path] + sys.path
        settings_module = __import__("settings_file_dir.deployed_settings")
        self.assertModuleHasSetting(settings_module.deployed_settings, "DATABASES")

if __name__ == '__main__':
    unittest.main()
